from pyPhasesRecordloaderSHHS.recordLoaders.RecordLoaderSHHS import RecordLoaderSHHS
from pyPhasesRecordloader.recordLoaders.CSVMetaLoader import CSVMetaLoader

from pyPhasesRecordloader.downloader.Downloader import Downloader


class RecordLoaderMrOS(RecordLoaderSHHS):
    def isVisit1(self, recordName):
        return recordName[:11] == "mros-visit1"

    def getFilePathSignal(self, recordName):
        dl = Downloader.get()
        pathEdfV1, _, pathEdfV2, _ = dl.basePathExtensionwise

        path = pathEdfV1 if self.isVisit1(recordName) else pathEdfV2

        return path + "/" + recordName + ".edf"

    def getFilePathAnnotation(self, recordId):
        isVisit1 = recordId[:11] == "mros-visit1"
        dl = Downloader.get()
        _, pathXmlV1, _, pathXmlV2 = dl.basePathExtensionwise

        path = pathXmlV1 if isVisit1 else pathXmlV2

        return path + "/" + recordId + "-nsrr.xml"

    def groupBy(self, name: str, recordIds, metadata=None):
        groupedRecords = {}
        if name == "patient":
            for recordId in recordIds:
                id = recordId[12:]

                if id not in groupedRecords:
                    groupedRecords[id] = []

                groupedRecords[id].append(recordId)
        return groupedRecords

    def getMetaData(self, recordName):
        metaData = super().getMetaData(recordName)
        metaData["recordId"] = recordName
        relevantRows = {
            # "nsrr_ahi_hp4u_aasm15",
            "nsrr_ahi_hp3r_aasm15": "ahi",
            "nsrr_phrnumar_f1": "arI",  # Arousal Index: Number of arousals per hour of sleep from polysomnography
            "nsrr_bmi": "bmi",  # Harmonized by the NSRR team to align with <a href=""https://github.com/UW-GAC/topmed-dcc-harmonized-phenotypes"" target=""_blank"">TOPMed</a>
            "nsrr_age": "age",
            "nsrr_age_gt89": "age_gt89",
            "nsrr_sex": "sex",
            "nsrr_race": "race",
            "nsrr_bp_diastolic": "bp_diastolic",
            "nsrr_bp_systolic": "bp_systolic",
        }
        visit = 1 if self.isVisit1(recordName) else 0
        csvLoader = CSVMetaLoader(
            f"{self.filePath}/datasets/mros-visit{visit}-harmonized-0.6.0.csv", idColumn="nsrrid", relevantRows=relevantRows
        )
        csvMetaData = csvLoader.getMetaData(recordName[12:].upper())
        metaData.update(csvMetaData)

        return metaData
